@@ -103,7 +103,7 @@ def capture_stdout_botrun_ask_folder(
     """
 
     if force:
-        client = drive_client_factory()
+        # client = drive_client_factory()
         # 使用 asyncio.run() 来运行异步方法
         # asyncio.run(client.delete_drive_folder(google_drive_folder_id))
         if os.path.exists(f"./data/{google_drive_folder_id}"):
@@ -123,12 +123,14 @@ def capture_stdout_botrun_ask_folder(
 
     qdrant_host = os.getenv("QDRANT_HOST", "qdrant")
     qdrant_port = os.getenv("QDRANT_PORT", 6333)
+    qdrant_api_key = os.getenv("QDRANT_API_KEY", 6333)
 
     collection_existed = asyncio.run(
         has_collection_in_qdrant(
             f"{google_drive_folder_id}",
             qdrant_host,
             qdrant_port,
+            qdrant_api_key,
         )
     )
     handle_downloaded_files_and_save_to_qdrant(
@@ -164,6 +166,7 @@ def handle_downloaded_files_and_save_to_qdrant(
 
     qdrant_host = os.getenv("QDRANT_HOST", "qdrant")
     qdrant_port = os.getenv("QDRANT_PORT", 6333)
+    qdrant_api_key = os.getenv("QDRANT_API_KEY", "")
 
     # collection_existed = asyncio.run(has_collection_in_qdrant(
     #     f"{google_drive_folder_id}",
@@ -180,6 +183,7 @@ def handle_downloaded_files_and_save_to_qdrant(
             f"{google_drive_folder_id}",
             qdrant_host,
             qdrant_port,
+            qdrant_api_key,
             force=force,
         )
     )
@@ -254,7 +258,7 @@ def botrun_ask_folder_separately(
 
         qdrant_host = os.getenv("QDRANT_HOST", "qdrant")
         qdrant_port = os.getenv("QDRANT_PORT", 6333)
-
+        qdrant_api_key = os.getenv("QDRANT_API_KEY", "")
         asyncio.run(
             embeddings_to_qdrant(
                 f"./data/{google_drive_folder_id}",
@@ -264,6 +268,7 @@ def botrun_ask_folder_separately(
                 f"{google_drive_folder_id}",
                 qdrant_host,
                 qdrant_port,
+                qdrant_api_key,
                 force=force,
             )
         )
