@@ -8,3 +8,5 @@ MAX_WORKERS = 50
 MAX_EMPTY_CHECKS = 10
 CHARS_PER_PAGE = 2000
 FAST_API_TIMEOUT = 600
+QUDRANT_ACTION_TIMEOUT = 60
+MAX_CONCURRENT_PROCESS_FILES = 50
