@@ -3,6 +3,8 @@ import os
 import shutil
 import uuid
 import asyncio
+import pytz
+from datetime import datetime
 from pathlib import Path
 from typing import List
 
@@ -125,27 +127,17 @@ async def init_drive_folder(folder_id: str, dic_result: dict):
     Initialize a DriveFolder object and save it as a JSON file.
     Also, add all DriveFiles to Firestore.
     """
-
     # Create DriveFiles from dic_result['items']
-    drive_files = [
-        DriveFile(
-            id=item["id"],
-            name=item["name"],
-            modifiedTime=item["modifiedTime"],
-            mimeType=item["mimeType"],
-            size=item.get("size", ""),
-            parent=item.get("parent", ""),
-            path=item.get("path", ""),
-            folder_id=folder_id,
-        )
-        for item in dic_result["items"]
-    ]
 
     # Create DriveFolder
     drive_folder = DriveFolder(
         id=dic_result["parent_id"],
         status=DriveFolderStatus.PROCESSING,
-        items=[file.id for file in drive_files],
+        items=[item["id"] for item in dic_result["items"]],
+        last_update_items_timestamp=datetime.now(pytz.timezone("Asia/Taipei")).strftime(
+            "%Y-%m-%d %H:%M:%S"
+        ),
+        last_update_items=[item["id"] for item in dic_result["items"]],
     )
 
     client = drive_client_factory()
@@ -153,16 +145,38 @@ async def init_drive_folder(folder_id: str, dic_result: dict):
     # Set the DriveFolder
     await client.set_drive_folder(drive_folder)
 
-    # Add all DriveFiles to Firestore
+    print(f"Initialized DriveFolder to Firestore.")
+
+
+async def set_drive_files(drive_files: list[DriveFile]):
+    client = drive_client_factory()
     for drive_file in drive_files:
         await client.set_drive_file(drive_file)
-        await client.update_drive_file_status_in_folder(
-            drive_file.folder_id, drive_file.id, drive_file.status
-        )
+        # await client.update_drive_file_status_in_folder(
+        #     drive_file.folder_id, drive_file.id, drive_file.status
+        # )
 
-    print(
-        f"Initialized DriveFolder and added {len(drive_files)} DriveFiles to Firestore."
-    )
+    print(f"Added {len(drive_files)} DriveFiles to Firestore.")
+
+
+async def get_drive_files_need_update(drive_files: list[DriveFile]):
+    """
+    Get the drive files that need to be updated.
+    """
+    client = drive_client_factory()
+    drive_files_need_update = []
+    for drive_file in drive_files:
+        ori_drive_file = await client.get_drive_file(drive_file.id)
+        if ori_drive_file is None:
+            drive_files_need_update.append(drive_file)
+            continue
+        if (
+            ori_drive_file.status != DriveFileStatus.EMBEDDED
+            or ori_drive_file.modifiedTime != drive_file.modifiedTime
+        ):
+            drive_files_need_update.append(drive_file)
+
+    return drive_files_need_update
 
 
 def update_new_items_to_drive_folder(folder_id: str, new_items: list):
@@ -204,8 +218,6 @@ async def update_drive_file_after_process_file(
         collection_id (str): The ID of the collection to update.
         lst_rag_metadata (List[RagMetadata]): List of processed file metadata.
     """
-    from botrun_ask_folder.drive_download import append_export_extension_to_path
-    from botrun_ask_folder.drive_download import truncate_filename
 
     # Get the collection status client
     client = drive_client_factory()
@@ -239,8 +251,8 @@ async def update_drive_file_after_process_file(
 
     drive_file.status = DriveFileStatus.SPLITTED
     await client.set_drive_file(drive_file)
-    await client.update_drive_file_status_in_folder(
-        drive_file.folder_id, drive_file.id, drive_file.status
-    )
+    # await client.update_drive_file_status_in_folder(
+    #     drive_file.folder_id, drive_file.id, drive_file.status
+    # )
 
     print(f"Updated drive file {drive_file.id} after processing file.")
