@@ -8,6 +8,7 @@ import json
 from litellm import aembedding, embedding
 from pathlib import Path
 from qdrant_client.http import models
+from botrun_ask_folder.constants import QUDRANT_ACTION_TIMEOUT
 from botrun_ask_folder.models.drive_file import DriveFile
 
 from botrun_ask_folder.models.rag_metadata import PAGE_NUMBER_NA
@@ -50,9 +51,9 @@ async def is_already_indexed(client, collection_name, text_content):
         return len(search_response[0]) > 0
     except Exception as e:
         print(
-            f"embeddings_to_qdrant.py, 發生例外錯誤，參數如下：\n- client: {client}\n- collection_name: {collection_name}\n- text_content: {text_content}"
+            f"embeddings_to_qdrant.py, is_already_indexed 發生例外錯誤，參數如下：\n- client: {client}\n- collection_name: {collection_name}\n- text_content: {text_content}"
         )
-        print(f"embeddings_to_qdrant.py, 錯誤訊息：{e}")
+        print(f"embeddings_to_qdrant.py, is_already_indexed 錯誤訊息：{e}")
         # 異常處理後，假設資料不存在
         return False
 
@@ -74,9 +75,9 @@ async def index_document(
         return True
     except Exception as e:
         print(
-            f"embeddings_to_qdrant.py, 上傳文件發生錯誤，參數如下：\n- client: {client}\n- collection_name: {collection_name}\n- document_data_payload: {document_data_payload}\n- embeddings: {embeddings}"
+            f"embeddings_to_qdrant.py, index_document 上傳文件發生錯誤，參數如下：\n- client: {client}\n- collection_name: {collection_name}\n- document_data_payload: {document_data_payload}\n- embeddings: {embeddings}"
         )
-        print(f"embeddings_to_qdrant.py, 錯誤訊息：{e}")
+        print(f"embeddings_to_qdrant.py, index_document 錯誤訊息：{e}")
         return False
 
 
@@ -93,7 +94,7 @@ def generate_embedding_sync(model, texts):
         print(
             f"embeddings_to_qdrant.py, 生成嵌入發生錯誤，參數如下：\n- model: {model}\n- texts: {texts}"
         )
-        print(f"embeddings_to_qdrant.py, 錯誤訊息：{e}")
+        print(f"embeddings_to_qdrant.py, generate_embedding_sync錯誤訊息：{e}")
         return {"data": [{"embedding": []}]}
 
 
@@ -108,7 +109,7 @@ async def generate_embedding_async(model, texts):
         print(
             f"embeddings_to_qdrant.py, 生成嵌入發生錯誤，參數如下：\n- model: {model}\n- texts: {texts}"
         )
-        print(f"embeddings_to_qdrant.py, 錯誤訊息：{e}")
+        print(f"embeddings_to_qdrant.py, generate_embedding_async 錯誤訊息：{e}")
         # 返回一個空的嵌入結果以避免中斷流程
         return {"data": [{"embedding": []}]}
 
@@ -122,7 +123,7 @@ async def process_file(
     counters,
     progress_bar,
     dic_metadata={},
-    splitted_items: dict[str, SplittedFile] = None,
+    # splitted_items: dict[str, SplittedFile] = None,
 ):
     async with semaphore:
         try:
@@ -165,15 +166,15 @@ async def process_file(
                                     "modifiedTime"
                                 ]
                             break
-                if splitted_items is not None and len(splitted_items) > 0:
-                    item = splitted_items.get(file_path.name, None)
-                    if item is not None:
-                        document_data_payload["google_file_id"] = item.id
-                        document_data_payload["gen_page_imgs"] = item.gen_page_imgs
-                        document_data_payload["page_number"] = item.page_number
-                        document_data_payload["sheet_name"] = item.sheet_name
-                        document_data_payload["file-upload-date"] = item.modified_time
-                        document_data_payload["ori_file_name"] = item.ori_file_name
+                # if splitted_items is not None and len(splitted_items) > 0:
+                #     item = splitted_items.get(file_path.name, None)
+                #     if item is not None:
+                #         document_data_payload["google_file_id"] = item.id
+                #         document_data_payload["gen_page_imgs"] = item.gen_page_imgs
+                #         document_data_payload["page_number"] = item.page_number
+                #         document_data_payload["sheet_name"] = item.sheet_name
+                #         document_data_payload["file-upload-date"] = item.modified_time
+                #         document_data_payload["ori_file_name"] = item.ori_file_name
 
                 await index_document(
                     client, collection_name, document_data_payload, embeddings
@@ -182,7 +183,7 @@ async def process_file(
             progress_bar.update(counters["skipped_count"] + counters["added_count"])
         except Exception as e:
             print(f"embeddings_to_qdrant.py, 處理文件發生錯誤，文件路徑：{file_path}")
-            print(f"embeddings_to_qdrant.py, 錯誤訊息：{e}")
+            print(f"embeddings_to_qdrant.py, process_file 錯誤訊息：{e}")
 
 
 async def process_split_file_for_distributed(
@@ -195,8 +196,11 @@ async def process_split_file_for_distributed(
     """
     這一段是給 fastapi 用的
     """
+    print(
+        f"embeddings_to_qdrant_distributed {split_file.id} start, ori_file_id: {split_file.file_id}"
+    )
     async with semaphore:
-        retry_count = 3
+        retry_count = 10
         success = False
         while not success and retry_count > 0:
             try:
@@ -204,12 +208,24 @@ async def process_split_file_for_distributed(
                     text_content = f.read()
 
                 if await is_already_indexed(client, collection_name, text_content):
-                    pass
+                    print(
+                        f"embeddings_to_qdrant_distributed {split_file.id} already indexed, return True,"
+                        f"ori_file_id: {split_file.file_id}"
+                    )
+                    return True
                 else:
+                    print(
+                        f"embeddings_to_qdrant_distributed {split_file.id} start embedding, "
+                        f"ori_file_id: {split_file.file_id}"
+                    )
                     embeddings_response = await generate_embedding_async(
                         model, [f"{split_file.save_path} {text_content}"]
                     )
                     embeddings = embeddings_response["data"][0]["embedding"]
+                    print(
+                        f"embeddings_to_qdrant_distributed {split_file.id} end embedding, "
+                        f"ori_file_id: {split_file.file_id}"
+                    )
 
                     document_data_payload = {
                         "text_content": text_content,
@@ -225,19 +241,35 @@ async def process_split_file_for_distributed(
                     success = await index_document(
                         client, collection_name, document_data_payload, embeddings
                     )
+                    print(
+                        f"embeddings_to_qdrant_distributed {split_file.id} end indexing success: {success}, "
+                        f"ori_file_id: {split_file.file_id}"
+                    )
                     if not success:
                         retry_count -= 1
                         await asyncio.sleep(3)
                         continue
                     else:
+                        print(
+                            f"embeddings_to_qdrant_distributed {split_file.id} return True, "
+                            f"ori_file_id: {split_file.file_id}"
+                        )
                         return True
-                return success
             except Exception as e:
                 print(
-                    f"embeddings_to_qdrant.py, 處理文件發生錯誤，文件路徑：{split_file.save_path}"
+                    f"embeddings_to_qdrant.py, 處理文件發生錯誤，文件路徑：{split_file.save_path}, ori_file_id: {split_file.file_id}"
                 )
-                print(f"embeddings_to_qdrant.py, 錯誤訊息：{e}")
-                return False
+                print(
+                    f"embeddings_to_qdrant.py, ori_file_id: {split_file.file_id} 錯誤訊息：{e}"
+                )
+                retry_count -= 1
+                await asyncio.sleep(3)
+                continue
+        print(
+            f"embeddings_to_qdrant_distributed {split_file.id} final success: {success}, "
+            f"ori_file_id: {split_file.file_id}"
+        )
+        return success
 
 
 def count_txt_files(input_folder):
@@ -251,13 +283,21 @@ def count_txt_files(input_folder):
         print(
             f"embeddings_to_qdrant.py, 計算文件數目發生錯誤，資料夾路徑：{input_folder}"
         )
-        print(f"embeddings_to_qdrant.py, 錯誤訊息：{e}")
+        print(f"embeddings_to_qdrant.py, count_txt_files 錯誤訊息：{e}")
     return total_files
 
 
-async def has_collection_in_qdrant(collection_name, qdrant_host, qdrant_port):
+async def has_collection_in_qdrant(
+    collection_name, qdrant_host, qdrant_por, qdrant_api_key
+):
     try:
-        client = qdrant_client.AsyncQdrantClient(qdrant_host, port=qdrant_port)
+        client = qdrant_client.AsyncQdrantClient(
+            qdrant_host,
+            port=qdrant_port,
+            timeout=QUDRANT_ACTION_TIMEOUT,
+            api_key=qdrant_api_key,
+            https=False,
+        )
         await client.get_collection(collection_name)
         return True
     except Exception as e:
@@ -272,10 +312,16 @@ async def embeddings_to_qdrant(
     collection_name,
     qdrant_host,
     qdrant_port,
+    qdrant_api_key,
     force=False,
 ):
     try:
-        client = qdrant_client.AsyncQdrantClient(qdrant_host, port=qdrant_port)
+        client = qdrant_client.AsyncQdrantClient(
+            qdrant_host,
+            port=qdrant_port,
+            api_key=qdrant_api_key,
+            https=False,
+        )
         if force:
             await remove_collection_with_client(client, collection_name)
         semaphore = asyncio.Semaphore(max_tasks)
@@ -302,9 +348,9 @@ async def embeddings_to_qdrant(
         progress_bar = EmojiProgressBar(count_txt_files(input_folder))
         int_txt_files_count = 0
         metadata_file_name = get_metadata_file_name(input_folder)
-        # dic_metadata = get_drive_download_metadata(input_folder)
-        drive_client = drive_client_factory()
-        split_files = await drive_client.get_split_files(collection_name)
+        dic_metadata = get_drive_download_metadata(input_folder)
+        # drive_client = drive_client_factory()
+        # split_files = await drive_client.get_split_files(collection_name)
 
         for root, dirs, files in os.walk(input_folder):
             for file in files:
@@ -320,8 +366,8 @@ async def embeddings_to_qdrant(
                         semaphore,
                         counters,
                         progress_bar,
-                        {},
-                        split_files,
+                        dic_metadata,
+                        # split_files,
                     )
                     tasks.append(task)
                     int_txt_files_count += 1
@@ -336,19 +382,25 @@ async def embeddings_to_qdrant(
         print(
             f"embeddings_to_qdrant.py, 處理資料夾發生錯誤，參數如下：\n- input_folder: {input_folder}\n- model: {model}\n- dimension: {dimension}\n- max_tasks: {max_tasks}\n- collection_name: {collection_name}\n- qdrant_host: {qdrant_host}\n- qdrant_port: {qdrant_port}"
         )
-        print(f"embeddings_to_qdrant.py, 錯誤訊息：{e}")
+        print(f"embeddings_to_qdrant.py, embeddings_to_qdrant 錯誤訊息：{e}")
 
 
 async def init_qdrant_collection(
     collection_name: str,
     dimension=3072,
     force=False,
+    qdrant_host="qdrant",
+    qdrant_port=6333,
+    qdrant_api_key="",
 ):
-    qdrant_host = os.getenv("QDRANT_HOST", "qdrant")
-    qdrant_port = os.getenv("QDRANT_PORT", 6333)
-
     try:
-        client = qdrant_client.AsyncQdrantClient(qdrant_host, port=qdrant_port)
+        client = qdrant_client.AsyncQdrantClient(
+            qdrant_host,
+            port=qdrant_port,
+            timeout=QUDRANT_ACTION_TIMEOUT,
+            api_key=qdrant_api_key,
+            https=False,
+        )
         if force:
             await remove_collection_with_client(client, collection_name)
 
@@ -356,6 +408,7 @@ async def init_qdrant_collection(
             f"{collection_name}",
             qdrant_host,
             qdrant_port,
+            qdrant_api_key,
         )
 
         if not collection_existed:
@@ -391,12 +444,19 @@ async def embeddings_to_qdrant_distributed(
     model="openai/text-embedding-3-large",
     dimension=3072,
     max_tasks=30,
+    qdrant_host="qdrant",
+    qdrant_port=6333,
+    qdrant_api_key="",
 ) -> bool:
-    qdrant_host = os.getenv("QDRANT_HOST", "qdrant")
-    qdrant_port = os.getenv("QDRANT_PORT", 6333)
-
     try:
-        client = qdrant_client.AsyncQdrantClient(qdrant_host, port=qdrant_port)
+        print(f"embeddings_to_qdrant_distributed function start {drive_file.id}")
+        client = qdrant_client.AsyncQdrantClient(
+            qdrant_host,
+            port=qdrant_port,
+            timeout=QUDRANT_ACTION_TIMEOUT,
+            api_key=qdrant_api_key,
+            https=False,
+        )
         semaphore = asyncio.Semaphore(max_tasks)
 
         tasks = []
@@ -414,25 +474,35 @@ async def embeddings_to_qdrant_distributed(
             )
             tasks.append(task)
 
+        print(f"embeddings_to_qdrant_distributed {drive_file.id} start embedding")
         results = await asyncio.gather(*tasks, return_exceptions=True)
+        print(f"embeddings_to_qdrant_distributed {drive_file.id} end embedding")
 
         success_count = sum(1 for result in results if result is True)
+        print(
+            f"embeddings_to_qdrant_distributed {drive_file.id} success count: {success_count}"
+        )
         failure_count = len(results) - success_count
+        print(
+            f"embeddings_to_qdrant_distributed {drive_file.id} failure count: {failure_count}"
+        )
         if failure_count > 0:
-            print(f"embeddings_to_qdrant_distributed {drive_file.name} failed")
-        print(f"embeddings_to_qdrant_distributed {drive_file.name} completed")
+            print(f"embeddings_to_qdrant_distributed {drive_file.id} failed")
+        print(f"embeddings_to_qdrant_distributed {drive_file.id} completed")
         print(f"Successful tasks: {success_count}")
         print(f"Failed tasks: {failure_count}")
 
         return failure_count == 0  # Return True if all tasks succeeded, False otherwise
     except Exception as e:
         print(
-            f"embeddings_to_qdrant_distributed, 處理資料夾發生錯誤，參數如下：\n- model: {model}\n- dimension: {dimension}\n- max_tasks: {max_tasks}\n- collection_name: {drive_file.folder_id}\n- qdrant_host: {qdrant_host}\n- qdrant_port: {qdrant_port}"
+            f"embeddings_to_qdrant_distributed, file_id: {drive_file.id} 處理資料夾發生錯誤，參數如下：\n- model: {model}\n- dimension: {dimension}\n- max_tasks: {max_tasks}\n- collection_name: {drive_file.folder_id}\n- qdrant_host: {qdrant_host}\n- qdrant_port: {qdrant_port}"
         )
         import traceback
 
         print(traceback.format_exc())
-        print(f"embeddings_to_qdrant_distributed, 錯誤訊息：{e}")
+        print(
+            f"embeddings_to_qdrant_distributed, file_id: {drive_file.id} 錯誤訊息：{e}"
+        )
         return False
 
 
@@ -475,7 +545,7 @@ if __name__ == "__main__":
         print(
             f"embeddings_to_qdrant.py, 主程序發生錯誤，參數如下：\n- input_folder: {args.input_folder}\n- model: {args.model}\n- dimension: {args.dimension}\n- max_tasks: {args.max_tasks}\n- collection_name: {args.collection_name}\n- qdrant_host: {qdrant_host}\n- qdrant_port: {qdrant_port}"
         )
-        print(f"embeddings_to_qdrant.py, 錯誤訊息：{e}")
+        print(f"embeddings_to_qdrant.py, main 錯誤訊息：{e}")
 
 """
 source venv/bin/activate
