@@ -1,10 +1,7 @@
-from fastapi import FastAPI, HTTPException, Query, APIRouter
+from fastapi import FastAPI, HTTPException, Query, APIRouter, Depends
 from fastapi.responses import StreamingResponse, Response, JSONResponse, FileResponse
 from urllib.parse import quote
 from fastapi.staticfiles import StaticFiles
-
-import functions_framework
-from flask import jsonify, Request, Response, stream_with_context
 from pydantic import BaseModel, Field
 import io
 import os
@@ -22,11 +19,10 @@ from botrun_ask_folder.google_drive_service import get_google_drive_service
 from botrun_ask_folder.models.drive_file import DriveFile, DriveFileStatus
 from botrun_ask_folder.models.drive_folder import DriveFolder, DriveFolderStatus
 from botrun_ask_folder.models.splitted_file import SplittedFile, SplittedFileStatus
-from botrun_ask_folder.run_split_txts import run_split_txts_for_distributed
-
-from botrun_ask_folder.drive_download import (
-    file_download_with_service,
+from botrun_ask_folder.process_folder_job import (
+    download_single_file_and_embed,
 )
+
 from botrun_ask_folder.services.drive.drive_factory import (
     drive_client_factory,
 )
@@ -35,11 +31,16 @@ from botrun_ask_folder.services.queue.queue_factory import (
 )
 from botrun_ask_folder.models.job_event import JobEvent
 from botrun_ask_folder.workers.worker_pool import worker_pool
+from google.cloud import run_v2
+from google.cloud.run_v2.types import RunJobRequest
+from dotenv import load_dotenv
+
+from botrun_ask_folder.fast_api.jwt_util import verify_token
+
+load_dotenv()
 
 router = APIRouter(prefix="/botrun_ask_folder", tags=["botrun_ask_folder"])
 
-# Mount static files
-import os
 
 current_dir = os.path.dirname(os.path.abspath(__file__))
 static_dir = os.path.join(current_dir, "static")
@@ -117,9 +118,12 @@ class FolderRequest(BaseModel):
     folder_id: str
     force: bool = False
     embed: bool = True
+    qdrant_host: str = "qdrant"
+    qdrant_port: int = 6333
+    qdrant_api_key: str = ""
 
 
-@router.post("/pub-process-folder")
+@router.post("/pub-process-folder", dependencies=[Depends(verify_token)])
 async def pub_process_folder(request: FolderRequest):
     print(f"Processing folder {request.folder_id} with force={request.force}")
 
@@ -144,40 +148,154 @@ async def pub_process_folder(request: FolderRequest):
     return {
         "message": f"Drive folder {request.folder_id} processing initiated",
         "status": "success",
-        # "items_enqueued": len(items),
+    }
+
+
+@router.post("/process-folder-job", dependencies=[Depends(verify_token)])
+async def process_folder_job(request: FolderRequest):
+    print(
+        f"Processing folder {request.folder_id} with force={request.force} using Cloud Run Job"
+    )
+
+    # Get the credentials from the key file
+    google_service_account_key_path = os.getenv(
+        "GOOGLE_APPLICATION_CREDENTIALS_FOR_FASTAPI",
+        "/app/keys/scoop-386004-d22d99a7afd9.json",
+    )
+    credentials = service_account.Credentials.from_service_account_file(
+        google_service_account_key_path,
+        scopes=["https://www.googleapis.com/auth/cloud-platform"],
+    )
+
+    # Create a Cloud Run Jobs client
+    client = run_v2.JobsClient(credentials=credentials)
+
+    # Get the project ID from the credentials
+    project = credentials.project_id
+
+    # Prepare the job request
+    job_name = f"projects/{project}/locations/{os.getenv('CLOUD_RUN_REGION', 'asia-east1')}/jobs/process-folder-job"
+
+    args = [
+        "--folder_id",
+        request.folder_id,
+        "--qdrant_host",
+        request.qdrant_host,
+        "--qdrant_port",
+        str(request.qdrant_port),
+        "--qdrant_api_key",
+        request.qdrant_api_key,
+    ]
+    if request.force:
+        args.append("--force")
+    if not request.embed:
+        args.append("--no-embed")
+    container_override = RunJobRequest.Overrides.ContainerOverride(
+        name="gcr.io/scoop-386004/botrun-ask-folder-job",
+        args=args,
+    )
+
+    job_overrides = RunJobRequest.Overrides(container_overrides=[container_override])
+
+    job_request = RunJobRequest(name=job_name, overrides=job_overrides)
+
+    print(
+        "start invoke Cloud Run Job process_folder_job with folder_id: ",
+        request.folder_id,
+    )
+    # 触发 Job
+    operation = client.run_job(request=job_request)
+
+    # 返回成功响应
+    return {
+        "message": "Job triggered successfully",
+        "job_id": operation.metadata.name,
+        "status": "success",
+    }
+
+
+@router.get("/heartbeat", dependencies=[Depends(verify_token)])
+async def heartbeat():
+    print("Triggering Cloud Run Job heartbeat check")
+
+    # Get the credentials from the key file
+    google_service_account_key_path = os.getenv(
+        "GOOGLE_APPLICATION_CREDENTIALS_FOR_FASTAPI",
+        "/app/keys/scoop-386004-d22d99a7afd9.json",
+    )
+    credentials = service_account.Credentials.from_service_account_file(
+        google_service_account_key_path,
+        scopes=["https://www.googleapis.com/auth/cloud-platform"],
+    )
+
+    # Create a Cloud Run Jobs client
+    client = run_v2.JobsClient(credentials=credentials)
+
+    # Get the project ID from the credentials
+    project = credentials.project_id
+
+    # Prepare the job request
+    job_name = f"projects/{project}/locations/{os.getenv('CLOUD_RUN_REGION', 'asia-east1')}/jobs/process-folder-job"
+
+    args = ["--folder_id", "heartbeat_check", "--heartbeat"]
+    container_override = RunJobRequest.Overrides.ContainerOverride(
+        name="gcr.io/scoop-386004/botrun-ask-folder-job",
+        args=args,
+    )
+
+    job_overrides = RunJobRequest.Overrides(container_overrides=[container_override])
+
+    job_request = RunJobRequest(name=job_name, overrides=job_overrides)
+
+    print("start invoke Cloud Run Job heartbeat check")
+    # 触发 Job
+    operation = client.run_job(request=job_request)
+
+    # 返回成功响应
+    return {
+        "message": "Heartbeat job triggered successfully",
+        "job_id": operation.metadata.name,
+        "status": "success",
     }
 
 
 class FolderStatusRequest(BaseModel):
     folder_id: str
+    # 因為 cloud run job 沒有立即執行，所以加入這個參數，如果偵測到 folder updated_at 比 action_started_at 舊，就回等待中
+    action_started_at: str = ""
 
 
-@router.post("/folder-status")
+@router.post("/folder-status", dependencies=[Depends(verify_token)])
 async def folder_status(request: FolderStatusRequest):
     folder_id = request.folder_id
-    print(f"[Folder {folder_id}] Received status request")
     client = drive_client_factory()
-    print(f"[Folder {folder_id}] Drive client factory created")
 
     try:
-        print(f"[Folder {folder_id}] Fetching folder")
         folder = await client.get_drive_folder(folder_id)
-        print(f"[Folder {folder_id}] Folder status: {folder.status}")
-
+        if request.action_started_at and folder.updated_at < request.action_started_at:
+            return {
+                "status": "WAITING",
+                "message": "Folder is waiting to be processed",
+            }
+        if folder is None:
+            return {
+                "status": "WAITING",
+                "message": f"Folder {folder_id} not found, Folder is waiting to be processed",
+            }
         total_files = len(folder.items)
-        embedded_files = sum(
-            1
-            for status in folder.file_statuses.values()
-            if status == DriveFileStatus.EMBEDDED
-        )
+        # embedded_files = sum(
+        #     1
+        #     for status in folder.file_statuses.values()
+        #     if status == DriveFileStatus.EMBEDDED
+        # )
 
         response = {
             "status": folder.status.value,
             "message": f"Folder {folder_id} status: {folder.status.value}",
             "updated_at": folder.updated_at,
             "total_files": total_files,
-            "embedded_files": embedded_files,
-            "processing_files": total_files - embedded_files,
+            # "last_update_items_timestamp": folder.last_update_items_timestamp,
+            # "last_update_items": folder.last_update_items,
         }
 
         if folder.status == DriveFolderStatus.DONE:
@@ -210,7 +328,7 @@ async def start_worker(request: FolderRequest):
     }
 
 
-@router.post("/complete-all-jobs")
+@router.post("/complete-all-jobs", dependencies=[Depends(verify_token)])
 async def complete_all_jobs():
     """
     清空所有 job queue 裡的 job，開發用
@@ -242,20 +360,24 @@ async def complete_all_jobs():
 
 
 class DriveFileRequest(BaseModel):
-    drive_file: str
+    file_id: str
     force: bool = False
     # 是否要 embed，這個是做壓測試的時候，可設成 false ，來節省 embed 的金額
     embed: bool = True
 
 
-@router.post("/process-file")
+@router.post("/process-file", dependencies=[Depends(verify_token)])
 async def process_file(request: DriveFileRequest):
-    drive_file = DriveFile.from_json(request.drive_file)
+    # drive_file = DriveFile.from_json(request.drive_file)
     force = request.force
     embed = request.embed
 
     try:
-        await _handle_download_and_embed(drive_file, force, embed)
+        drive_client = drive_client_factory()
+        drive_file = await drive_client.get_drive_file(request.file_id)
+        download_single_file_and_embed(
+            drive_file, get_google_drive_service(), force, embed
+        )
         return {
             "status": "success",
             "message": f"File {drive_file.id} processed successfully",
@@ -264,76 +386,78 @@ async def process_file(request: DriveFileRequest):
         raise HTTPException(status_code=500, detail=str(e))
 
 
-async def _handle_download_and_embed(
-    drive_file: DriveFile, force: bool, embed: bool = True
-):
-    folder_path = "./data"
-    print(f"Downloading file: {drive_file.id}")
-    drive_file = file_download_with_service(
-        get_google_drive_service(), drive_file, folder_path, force=force
-    )
-    drive_client = drive_client_factory()
-    await drive_client.set_drive_file(drive_file)
-    await drive_client.update_drive_file_status_in_folder(
-        drive_file.folder_id, drive_file.id, drive_file.status
-    )
-    await run_split_txts_for_distributed(drive_file, force=force)
-    print(f"File: {drive_file.id} splitted")
+class FolderLastUpdateRequest(BaseModel):
+    folder_id: str
 
-    if embed:
-        embed_success = False
-        try:
-            embed_success = await embeddings_to_qdrant_distributed(drive_file)
-        except Exception as e:
-            import traceback
 
-            traceback.print_exc()
-            print(f"Embedding 失敗，錯誤訊息：{e}")
-        if embed_success:
-            drive_file.status = DriveFileStatus.EMBEDDED
-            await drive_client.set_drive_file(drive_file)
-            await drive_client.update_drive_file_status_in_folder(
-                drive_file.folder_id, drive_file.id, drive_file.status
-            )
+@router.post("/folder-last-update-items", dependencies=[Depends(verify_token)])
+async def folder_last_update_items(request: FolderLastUpdateRequest):
+    folder_id = request.folder_id
+    client = drive_client_factory()
 
-            await _finalize_embed(drive_file)
-    else:
-        for split_id in drive_file.splitted_files:
-            split_file = await drive_client.get_splitted_file(split_id)
-            split_file.status = SplittedFileStatus.EMBEDDED
-            await drive_client.set_splitted_file(split_file)
-        drive_file.status = DriveFileStatus.EMBEDDED
-        await drive_client.set_drive_file(drive_file)
-        await drive_client.update_drive_file_status_in_folder(
-            drive_file.folder_id, drive_file.id, drive_file.status
+    try:
+        folder = await client.get_drive_folder(folder_id)
+        if folder is None:
+            return {
+                "status": "NOT_FOUND",
+                "message": f"Folder {folder_id} not found",
+            }
+
+        response = {
+            "status": "SUCCESS",
+            "message": f"Last update items for folder {folder_id}",
+            "last_update_items_timestamp": folder.last_update_items_timestamp,
+            "last_update_items": folder.last_update_items,
+        }
+
+        print(f"[Folder {folder_id}] Last Update Items Response: {response}")
+        return response
+
+    except Exception as e:
+        print(f"[Folder {folder_id}] Error in folder_last_update_items: {str(e)}")
+        raise HTTPException(
+            status_code=500,
+            detail=f"Error processing folder last update items: {str(e)}",
         )
-        await _finalize_embed(drive_file)
 
 
-async def _finalize_embed(drive_file: DriveFile):
-    drive_client = drive_client_factory()
-    for item in drive_file.splitted_files:
-        split_file = await drive_client.get_splitted_file(item)
-        split_file.status = SplittedFileStatus.EMBEDDED
-        await drive_client.set_splitted_file(split_file)
-        if split_file.save_path:
-            try:
-                os.remove(split_file.save_path)
-            except Exception as e:
-                print(f"Error removing split file {split_file.id} save path: {e}")
-    drive_folder = await drive_client.get_drive_folder(drive_file.folder_id)
-    all_files_embedded = True
-    for drive_file in drive_folder.items:
-        drive_file = await drive_client_factory().get_drive_file(drive_file)
-        if drive_file.status != DriveFileStatus.EMBEDDED:
-            all_files_embedded = False
-            break
-    if all_files_embedded:
-        drive_folder.status = DriveFolderStatus.DONE
-        await drive_client.set_drive_folder(drive_folder)
-    if drive_file.save_path:
-        try:
-            os.remove(drive_file.save_path)
-        except Exception as e:
-            print(f"Error removing drive file {drive_file.id} save path: {e}")
-    print(f"Finalize embed for drive file {drive_file.id}")
+# 添加新的請求模型
+class FileStatusRequest(BaseModel):
+    file_id: str
+
+
+# 添加新的端點
+@router.post("/file-status", dependencies=[Depends(verify_token)])
+async def file_status(request: FileStatusRequest):
+    file_id = request.file_id
+    client = drive_client_factory()
+
+    try:
+        drive_file = await client.get_drive_file(file_id)
+        if drive_file is None:
+            return {
+                "status": "NOT_FOUND",
+                "message": f"File {file_id} not found",
+            }
+
+        response = {
+            "status": "SUCCESS",
+            "message": f"Status for file {file_id}",
+            "file_status": drive_file.status.value,
+            "file_name": drive_file.name,
+            "updated_at": drive_file.updated_at,
+            "folder_id": drive_file.folder_id,
+            "mime_type": drive_file.mimeType,
+            "modified_time": drive_file.modifiedTime,
+            "size": drive_file.size,
+        }
+
+        print(f"[File {file_id}] File Status Response: {response}")
+        return response
+
+    except Exception as e:
+        print(f"[File {file_id}] Error in file_status: {str(e)}")
+        raise HTTPException(
+            status_code=500,
+            detail=f"Error processing file status: {str(e)}",
+        )
