@@ -19,9 +19,9 @@ from dotenv import load_dotenv
 
 class BotrunLLMError(Exception):  # use this for all your exceptions
     def __init__(
-            self,
-            status_code,
-            message,
+        self,
+        status_code,
+        message,
     ):
         self.status_code = status_code
         self.message = message
@@ -31,14 +31,33 @@ class BotrunLLMError(Exception):  # use this for all your exceptions
 
 
 class BotrunLLMParams:
-    def __init__(self, qdrant_host, qdrant_port, collection_name, user_input,
-                 embedding_model, top_k, notice_prompt,
-                 chat_model, hnsw_ef, file_path_field, text_content_field,
-                 google_file_id_field, page_number_field, gen_page_imgs_field,
-                 ori_file_name_field, sheet_name_field, file_upload_date_field):
+    def __init__(
+        self,
+        qdrant_host,
+        qdrant_port,
+        qdrant_api_key,
+        collection_name,
+        chat_history,
+        user_input,
+        embedding_model,
+        top_k,
+        notice_prompt,
+        chat_model,
+        hnsw_ef,
+        file_path_field,
+        text_content_field,
+        google_file_id_field,
+        page_number_field,
+        gen_page_imgs_field,
+        ori_file_name_field,
+        sheet_name_field,
+        file_upload_date_field,
+    ):
         self.qdrant_host = qdrant_host
         self.qdrant_port = qdrant_port
+        self.qdrant_api_key = qdrant_api_key
         self.collection_name = collection_name
+        self.chat_history = chat_history
         self.user_input = user_input
         self.embedding_model = embedding_model
         self.top_k = top_k
@@ -61,15 +80,19 @@ class BotrunLLM(CustomLLM):
         model = kwargs.get("model", "")
         messages = kwargs.get("messages", [])
         botrun_name = model.split("-")[-1]
-        folder_id = os.environ.get('GOOGLE_DRIVE_BOTS_FOLDER_ID')
+        folder_id = os.environ.get("GOOGLE_DRIVE_BOTS_FOLDER_ID")
         if not folder_id:
-            raise BotrunLLMError(status_code=500, message="GOOGLE_DRIVE_BOTS_FOLDER_ID environment variable is not set")
+            raise BotrunLLMError(
+                status_code=500,
+                message="GOOGLE_DRIVE_BOTS_FOLDER_ID environment variable is not set",
+            )
 
         # 從 botrun 檔案讀取 notice_prompt
         # chat_model = "openai/gpt-4o-2024-08-06"
         try:
-            notice_prompt, collection_name, chat_model = read_notice_prompt_and_collection_from_botrun(botrun_name,
-                                                                                                       folder_id)
+            notice_prompt, collection_name, chat_model = (
+                read_notice_prompt_and_collection_from_botrun(botrun_name, folder_id)
+            )
             # print(f"notice_prompt: {notice_prompt}")
             # print(f"collection_name: {collection_name}")
             # print(f"_get_botrun_llm_params chat_model: {chat_model}")
@@ -88,16 +111,34 @@ class BotrunLLM(CustomLLM):
         file_upload_date_field = "file-upload-date"
         qdrant_host = os.getenv("QDRANT_HOST", "localhost")
         qdrant_port = os.getenv("QDRANT_PORT", 6333)
-        embedding_model = 'openai/text-embedding-3-large'
+        qdrant_api_key = os.getenv("QDRANT_API_KEY", "")
+        embedding_model = "openai/text-embedding-3-large"
         top_k = 6
         hnsw_ef = 256
 
-        user_input = messages[-1]['content'] if messages else ""
-        return BotrunLLMParams(qdrant_host, qdrant_port, collection_name, user_input,
-                               embedding_model, top_k, notice_prompt,
-                               chat_model, hnsw_ef, file_path_field, text_content_field,
-                               google_file_id_field, page_number_field, gen_page_imgs_field,
-                               ori_file_name_field, sheet_name_field, file_upload_date_field)
+        chat_history = messages[:-1] if len(messages) > 1 else []
+        user_input = messages[-1]["content"] if messages else ""
+        return BotrunLLMParams(
+            qdrant_host,
+            qdrant_port,
+            qdrant_api_key,
+            collection_name,
+            chat_history,
+            user_input,
+            embedding_model,
+            top_k,
+            notice_prompt,
+            chat_model,
+            hnsw_ef,
+            file_path_field,
+            text_content_field,
+            google_file_id_field,
+            page_number_field,
+            gen_page_imgs_field,
+            ori_file_name_field,
+            sheet_name_field,
+            file_upload_date_field,
+        )
 
     def completion(self, *args, **kwargs) -> litellm.ModelResponse:
         print("BotrunLLM.completion")
@@ -111,15 +152,19 @@ class BotrunLLM(CustomLLM):
         stream = kwargs.get("stream", False)
         model = kwargs.get("model", "")
 
-        result = await self._generate_complete(self._get_botrun_llm_params(*args, **kwargs))
+        result = await self._generate_complete(
+            self._get_botrun_llm_params(*args, **kwargs)
+        )
         return ModelResponse(
             id=f"botrun-{uuid.uuid4()}",
-            choices=[{
-                "message": {"role": "assistant", "content": result},
-                "finish_reason": "stop"
-            }],
+            choices=[
+                {
+                    "message": {"role": "assistant", "content": result},
+                    "finish_reason": "stop",
+                }
+            ],
             model=model,
-            usage={"prompt_tokens": 0, "completion_tokens": 0, "total_tokens": 0}
+            usage={"prompt_tokens": 0, "completion_tokens": 0, "total_tokens": 0},
         )
 
     async def _sync_to_async_generator(self, sync_gen):
@@ -127,52 +172,64 @@ class BotrunLLM(CustomLLM):
             yield item
             await asyncio.sleep(0)  # 讓出控制權，允許其他協程運行
 
-    async def _generate_stream(self, botrun_llm_params: BotrunLLMParams, model: str) -> AsyncIterator[ModelResponse]:
+    async def _generate_stream(
+        self, botrun_llm_params: BotrunLLMParams, model: str
+    ) -> AsyncIterator[ModelResponse]:
         # print("BotrunLLM._generate_stream model:", model)
-        async for fragment in self._sync_to_async_generator(query_qdrant_and_llm(
-                botrun_llm_params.qdrant_host, botrun_llm_params.qdrant_port,
-                botrun_llm_params.collection_name, botrun_llm_params.user_input,
-                botrun_llm_params.embedding_model, botrun_llm_params.top_k,
-                botrun_llm_params.notice_prompt, botrun_llm_params.chat_model,
-                botrun_llm_params.hnsw_ef, botrun_llm_params.file_path_field,
-                botrun_llm_params.text_content_field, botrun_llm_params.google_file_id_field,
-                botrun_llm_params.page_number_field, botrun_llm_params.gen_page_imgs_field,
-                botrun_llm_params.ori_file_name_field, botrun_llm_params.sheet_name_field,
-                botrun_llm_params.file_upload_date_field, include_ref_page=False)):
+        async for fragment in self._sync_to_async_generator(
+            query_qdrant_and_llm(
+                botrun_llm_params.qdrant_host,
+                botrun_llm_params.qdrant_port,
+                botrun_llm_params.collection_name,
+                botrun_llm_params.user_input,
+                botrun_llm_params.embedding_model,
+                botrun_llm_params.top_k,
+                botrun_llm_params.notice_prompt,
+                botrun_llm_params.chat_model,
+                botrun_llm_params.hnsw_ef,
+                botrun_llm_params.file_path_field,
+                botrun_llm_params.text_content_field,
+                botrun_llm_params.google_file_id_field,
+                botrun_llm_params.page_number_field,
+                botrun_llm_params.gen_page_imgs_field,
+                botrun_llm_params.ori_file_name_field,
+                botrun_llm_params.sheet_name_field,
+                botrun_llm_params.file_upload_date_field,
+                include_ref_page=False,
+                chat_history=botrun_llm_params.chat_history,
+                qdrant_api_key=botrun_llm_params.qdrant_api_key,
+            )
+        ):
             # print("BotrunLLM._generate_stream fragment:", fragment)
             yield ModelResponse(
                 id=f"botrun-chunk-{uuid.uuid4()}",
-                choices=[{
-                    "delta": {"content": fragment},
-                    "finish_reason": None
-                }],
+                choices=[{"delta": {"content": fragment}, "finish_reason": None}],
                 model=f"botrun/{model}",
                 stream=True,
             )
         # print("BotrunLLM._generate_stream finish")
         yield ModelResponse(
             id=f"botrun-chunk-{uuid.uuid4()}",
-            choices=[{
-                "delta": {"content": ""},
-                "finish_reason": "stop"
-            }],
+            choices=[{"delta": {"content": ""}, "finish_reason": "stop"}],
             model=f"botrun/{model}",
-            stream=True
+            stream=True,
         )
 
-    async def _generate_generic_stream_chunk(self, botrun_llm_params: BotrunLLMParams, model: str) -> AsyncIterator[
-        GenericStreamingChunk]:
+    async def _generate_generic_stream_chunk(
+        self, botrun_llm_params: BotrunLLMParams, model: str
+    ) -> AsyncIterator[GenericStreamingChunk]:
         # print("BotrunLLM._generate_generic_stream_chunk model:", model)
         index = 0
+        messages = botrun_llm_params.chat_history + [
+            {"role": "user", "content": botrun_llm_params.user_input},
+            {"role": "system", "content": botrun_llm_params.notice_prompt},
+        ]
         if botrun_llm_params.collection_name is None:
             # 使用一般的 litellm streaming
             response = await litellm.acompletion(
                 model=botrun_llm_params.chat_model,
-                messages=[
-                    {"role": "user", "content": botrun_llm_params.user_input},
-                    {"role": "system", "content": botrun_llm_params.notice_prompt},
-                ],
-                stream=True
+                messages=messages,
+                stream=True,
             )
 
             async for chunk in response:
@@ -184,7 +241,11 @@ class BotrunLLM(CustomLLM):
                         is_finished=False,
                         text=content,
                         tool_use=None,
-                        usage={"completion_tokens": 0, "prompt_tokens": 0, "total_tokens": 0},
+                        usage={
+                            "completion_tokens": 0,
+                            "prompt_tokens": 0,
+                            "total_tokens": 0,
+                        },
                     )
                     index += 1
 
@@ -195,19 +256,37 @@ class BotrunLLM(CustomLLM):
                 is_finished=True,
                 text="",
                 tool_use=None,
-                usage={"completion_tokens": 0, "prompt_tokens": 0, "total_tokens": 0},  # 您可能需要累積實際的使用量
+                usage={
+                    "completion_tokens": 0,
+                    "prompt_tokens": 0,
+                    "total_tokens": 0,
+                },  # 您可能需要累積實際的使用量
             )
         else:
-            async for fragment in self._sync_to_async_generator(query_qdrant_and_llm(
-                    botrun_llm_params.qdrant_host, botrun_llm_params.qdrant_port,
-                    botrun_llm_params.collection_name, botrun_llm_params.user_input,
-                    botrun_llm_params.embedding_model, botrun_llm_params.top_k,
-                    botrun_llm_params.notice_prompt, botrun_llm_params.chat_model,
-                    botrun_llm_params.hnsw_ef, botrun_llm_params.file_path_field,
-                    botrun_llm_params.text_content_field, botrun_llm_params.google_file_id_field,
-                    botrun_llm_params.page_number_field, botrun_llm_params.gen_page_imgs_field,
-                    botrun_llm_params.ori_file_name_field, botrun_llm_params.sheet_name_field,
-                    botrun_llm_params.file_upload_date_field, include_ref_page=False)):
+            async for fragment in self._sync_to_async_generator(
+                query_qdrant_and_llm(
+                    botrun_llm_params.qdrant_host,
+                    botrun_llm_params.qdrant_port,
+                    botrun_llm_params.collection_name,
+                    botrun_llm_params.user_input,
+                    botrun_llm_params.embedding_model,
+                    botrun_llm_params.top_k,
+                    botrun_llm_params.notice_prompt,
+                    botrun_llm_params.chat_model,
+                    botrun_llm_params.hnsw_ef,
+                    botrun_llm_params.file_path_field,
+                    botrun_llm_params.text_content_field,
+                    botrun_llm_params.google_file_id_field,
+                    botrun_llm_params.page_number_field,
+                    botrun_llm_params.gen_page_imgs_field,
+                    botrun_llm_params.ori_file_name_field,
+                    botrun_llm_params.sheet_name_field,
+                    botrun_llm_params.file_upload_date_field,
+                    include_ref_page=False,
+                    chat_history=botrun_llm_params.chat_history,
+                    qdrant_api_key=botrun_llm_params.qdrant_api_key,
+                )
+            ):
                 # print("BotrunLLM._generate_stream fragment:", fragment)
                 yield GenericStreamingChunk(
                     finish_reason=None,
@@ -215,7 +294,11 @@ class BotrunLLM(CustomLLM):
                     is_finished=False,
                     text=fragment,
                     tool_use=None,
-                    usage={"completion_tokens": 10, "prompt_tokens": 20, "total_tokens": 30},
+                    usage={
+                        "completion_tokens": 10,
+                        "prompt_tokens": 20,
+                        "total_tokens": 30,
+                    },
                 )
                 index += 1
             # print("BotrunLLM._generate_stream finish")
@@ -225,7 +308,11 @@ class BotrunLLM(CustomLLM):
                 is_finished=True,
                 text="",
                 tool_use=None,
-                usage={"completion_tokens": 10, "prompt_tokens": 20, "total_tokens": 30},
+                usage={
+                    "completion_tokens": 10,
+                    "prompt_tokens": 20,
+                    "total_tokens": 30,
+                },
             )
 
     async def _generate_complete(self, *args) -> str:
@@ -233,24 +320,40 @@ class BotrunLLM(CustomLLM):
         botrun_llm_params = args[0]
         if botrun_llm_params.collection_name is None:
             # 使用一般的 litellm completion
+            messages = botrun_llm_params.chat_history + [
+                {"role": "user", "content": botrun_llm_params.user_input},
+                {"role": "system", "content": botrun_llm_params.notice_prompt},
+            ]
             response = await litellm.acompletion(
                 model=botrun_llm_params.chat_model,
-                messages=[{"role": "user", "content": botrun_llm_params.user_input},
-                          {"role": "system", "content": botrun_llm_params.notice_prompt}
-                          ]
+                messages=messages,
             )
             return response.choices[0].message.content
         else:
-            async for fragment in self._sync_to_async_generator(query_qdrant_and_llm(
-                    botrun_llm_params.qdrant_host, botrun_llm_params.qdrant_port,
-                    botrun_llm_params.collection_name, botrun_llm_params.user_input,
-                    botrun_llm_params.embedding_model, botrun_llm_params.top_k,
-                    botrun_llm_params.notice_prompt, botrun_llm_params.chat_model,
-                    botrun_llm_params.hnsw_ef, botrun_llm_params.file_path_field,
-                    botrun_llm_params.text_content_field, botrun_llm_params.google_file_id_field,
-                    botrun_llm_params.page_number_field, botrun_llm_params.gen_page_imgs_field,
-                    botrun_llm_params.ori_file_name_field, botrun_llm_params.sheet_name_field,
-                    botrun_llm_params.file_upload_date_field, include_ref_page=False)):
+            async for fragment in self._sync_to_async_generator(
+                query_qdrant_and_llm(
+                    botrun_llm_params.qdrant_host,
+                    botrun_llm_params.qdrant_port,
+                    botrun_llm_params.collection_name,
+                    botrun_llm_params.user_input,
+                    botrun_llm_params.embedding_model,
+                    botrun_llm_params.top_k,
+                    botrun_llm_params.notice_prompt,
+                    botrun_llm_params.chat_model,
+                    botrun_llm_params.hnsw_ef,
+                    botrun_llm_params.file_path_field,
+                    botrun_llm_params.text_content_field,
+                    botrun_llm_params.google_file_id_field,
+                    botrun_llm_params.page_number_field,
+                    botrun_llm_params.gen_page_imgs_field,
+                    botrun_llm_params.ori_file_name_field,
+                    botrun_llm_params.sheet_name_field,
+                    botrun_llm_params.file_upload_date_field,
+                    include_ref_page=False,
+                    chat_history=botrun_llm_params.chat_history,
+                    qdrant_api_key=botrun_llm_params.qdrant_api_key,
+                )
+            ):
                 result += fragment
         return result
 
@@ -261,7 +364,9 @@ class BotrunLLM(CustomLLM):
     async def astreaming(self, *args, **kwargs) -> AsyncIterator[GenericStreamingChunk]:
         model = kwargs.get("model", "")
         print("BotrunLLM.astreaming model:", model)
-        async for chunk in self._generate_generic_stream_chunk(self._get_botrun_llm_params(*args, **kwargs), model):
+        async for chunk in self._generate_generic_stream_chunk(
+            self._get_botrun_llm_params(*args, **kwargs), model
+        ):
             yield chunk
 
     def _sync_generator(self, async_gen):
@@ -280,27 +385,39 @@ def read_botrun_content(botrun_name: str, folder_id: str) -> str:
         # 建立 Google Drive API 客戶端
         service_account_file = os.getenv("GOOGLE_APPLICATION_CREDENTIALS", "")
         if not service_account_file:
-            raise Exception("GOOGLE_APPLICATION_CREDENTIALS environment variable is not set")
-        credentials = service_account.Credentials.from_service_account_file(service_account_file)
-        service = build('drive', 'v3', credentials=credentials)
+            raise Exception(
+                "GOOGLE_APPLICATION_CREDENTIALS environment variable is not set"
+            )
+        credentials = service_account.Credentials.from_service_account_file(
+            service_account_file
+        )
+        service = build("drive", "v3", credentials=credentials)
 
         # 搜尋 .botrun 檔案
         file_query = f"'{folder_id}' in parents and name='{botrun_name}.botrun'"
-        file_results = service.files().list(q=file_query, spaces='drive', fields='files(id, mimeType)').execute()
-        files = file_results.get('files', [])
+        file_results = (
+            service.files()
+            .list(q=file_query, spaces="drive", fields="files(id, mimeType)")
+            .execute()
+        )
+        files = file_results.get("files", [])
 
         if not files:
-            raise Exception(f"File '{botrun_name}.botrun' not found in the specified folder")
+            raise Exception(
+                f"File '{botrun_name}.botrun' not found in the specified folder"
+            )
 
-        file_id = files[0]['id']
-        mime_type = files[0]['mimeType']
+        file_id = files[0]["id"]
+        mime_type = files[0]["mimeType"]
 
         # 讀取檔案內容
-        if mime_type == 'application/vnd.google-apps.document':
+        if mime_type == "application/vnd.google-apps.document":
             # 如果是 Google Docs 文件，使用 export 方法
-            request = service.files().export_media(fileId=file_id, mimeType='text/plain')
+            request = service.files().export_media(
+                fileId=file_id, mimeType="text/plain"
+            )
             file_content = request.execute()
-            return file_content.decode('utf-8')
+            return file_content.decode("utf-8")
         else:
             # 對於其他類型的文件，使用 get_media 方法
             request = service.files().get_media(fileId=file_id)
@@ -309,11 +426,12 @@ def read_botrun_content(botrun_name: str, folder_id: str) -> str:
             done = False
             while done is False:
                 _, done = downloader.next_chunk()
-            return file_content.getvalue().decode('utf-8')
+            return file_content.getvalue().decode("utf-8")
 
     except HttpError as error:
         raise Exception(f"An error occurred: {error}")
 
+
 def extract_model_name(content: str) -> str:
     """
     從給定的字符串中提取模型名稱。
@@ -322,12 +440,13 @@ def extract_model_name(content: str) -> str:
     :param content: 包含模型名稱的字符串
     :return: 提取出的模型名稱，如果沒有找到則返回空字符串
     """
-    pattern = r'(?i)model\s*=\s*([\w-]+/[\w-]+(?:-[\w-]+)*)'
+    pattern = r"(?i)model\s*=\s*([\w-]+/[\w-]+(?:-[\w-]+)*)"
     match = re.search(pattern, content)
     if match:
         return match.group(1)
     return ""
 
+
 def read_notice_prompt_and_collection_from_botrun(botrun_name, folder_id):
     """
     從 botrun 檔案讀取 notice_prompt 和 collection_name
@@ -342,7 +461,7 @@ def read_notice_prompt_and_collection_from_botrun(botrun_name, folder_id):
 
     chat_model = "openai/gpt-4o-2024-08-06"
     tmp_model = extract_model_name(lines[0])
-    if tmp_model!="":
+    if tmp_model != "":
         chat_model = tmp_model
 
     # 分割內容
@@ -358,6 +477,7 @@ def read_notice_prompt_and_collection_from_botrun(botrun_name, folder_id):
 
     # 解析 JSON 部分以獲取 collection_name
     import json
+
     try:
         rag_config = json.loads(parts[1].split("@end")[0])
         collection_name = rag_config.get("collection_name")
