@@ -11,6 +11,7 @@ from typing import List, Optional, Dict, Union
 
 from botrun_ask_folder.generate_pages_html import generate_pdf_gallery_html
 from botrun_ask_folder.models.rag_metadata import PAGE_NUMBER_NA
+from botrun_ask_folder.models.reference import Reference
 from .embeddings_to_qdrant import generate_embedding_sync
 from litellm import ModelResponse
 
@@ -38,80 +39,7 @@ DEFAULT_NOTICE_PROMPT = """
 5. 回答時也記下基於「知識庫」回覆時所使用的區塊，做為以下 沉穩參考來源 的依據
 6. 結尾的部份我會自行加入參考資料，你不要自己加參考資料，不要自己加參考來源，不要自己加參考文件等等
 7. 結尾的總結只需要講一次，不要講兩次
-8. 回答的最後面請加上這行之下的內容 (請務必包含這個用雙尖括號的文字<<Botrun.ai波特人的沉穩參考來源>>)：
-<<Botrun.ai波特人的沉穩參考來源>>
-下面接一個 JSON，格式如下，不要使用 markdown的語法包覆：
-{
-    "references":[
-        // 這裡必須填入至少一個引用對象
-    ]
-}
 
-填充 references array內容的步驟：
-1. 仔細閱讀 RAG「知識庫」中的所有內容。
-2. 識別與回答相關的文件。
-3. 從這些文件中提取必要的信息。
-4. 為每個相關文件創建一個引用對象。
-5. 將所有創建的引用對象添加到 references 數組中。
-
-每個引用對象的格式如下：
-{
-    "ori_file_name":"[從 RAG 知識庫的 ori_file_name tag 中準確複製，只能是基於「知識庫」回覆時所使用的區塊中的內容]",
-    "page_number":"[從 RAG 知識庫的 page_number tag 中準確複製，必須是數字字串，只能是基於「知識庫」回覆時所使用的區塊中的內容]",
-    "sheet_name":"[從 RAG 知識庫的 sheet_name tag 中準確複製，只能是基於「知識庫」回覆時所使用的區塊中的內容]",
-    "google_file_id":"[從 RAG 知識庫的 google_file_id tag 中準確複製，只能是基於「知識庫」回覆時所使用的區塊中的內容]",
-    "gen_page_imgs":"[從 RAG 知識庫的 gen_page_imgs tag 中準確複製，只能是基於「知識庫」回覆時所使用的區塊中的內容]",
-    "collection_name":"[從 RAG 知識庫的 collection_name tag 中準確複製，只能是基於「知識庫」回覆時所使用的區塊中的內容]"
-}
-
-
-警告：
-- 不得生成任何不在 RAG 知識庫中的信息。
-- 如果找不到某個欄位的信息，請將該欄位設置為空字符串，而不是省略該欄位。
-- JSON 上下不要使用 markdown的語法包覆
-- 結尾只需要一個 JSON，不要傳多個
-
-
-正確示例：
-{
-    "references":[
-        {
-            "ori_file_name":"RAG知識庫有參考到的檔案1.pdf",
-            "page_number":"6",
-            "sheet_name":"",
-            "google_file_id":"16fje29lsfjo_DXV_Ako1fjowiLKJIOkljiofelk",
-            "gen_page_imgs":"True",
-            "collection_name":"1jooij30kjfoiejAFEsofeFiqjofoe"
-        },
-        {
-            "ori_file_name":"RAG知識庫有參考到的檔案1.pdf",
-            "page_number":"8",
-            "sheet_name":"",
-            "google_file_id":"16fje29lsfjo_DXV_Ako1fjowiLKJIOkljiofelk",
-            "gen_page_imgs":"True",
-            "collection_name":"1jooij30kjfoiejAFEsofeFiqjofoe"
-        },
-        {
-            "ori_file_name":"RAG知識庫有參考到的檔案2.pdf",
-            "page_number":"24",
-            "sheet_name":"",
-            "google_file_id":"26fjefoiewoiIJIOEF8jeowjioIU7e",
-            "gen_page_imgs":"True",
-            "collection_name":"1jooij30kjfoiejAFEsofeFiqjofoe"
-        },
-        {
-            "ori_file_name":"RAG知識庫有參考到的檔案3.xlsx",
-            "page_number":"",
-            "sheet_name":"sheet_1",
-            "google_file_id":"1foiej484loe65jfieowsJIE76f2",
-            "gen_page_imgs":"False",
-            "collection_name":"1jooij30kjfoiejAFEsofeFiqjofoe"
-        }
-    ]
-}
-
-
-請確保每個引用都遵循這個準確性標準。
 """
 
 SPECIAL_STRING = "<<Botrun.ai波特人的沉穩參考來源>>"
@@ -133,9 +61,12 @@ def query_qdrant_knowledge_base(
     ori_file_name_field="ori_file_name",
     sheet_name_field="sheet_name",
     file_upload_date_field="file-upload-date",
+    qdrant_api_key="",
 ) -> str:
     custom_log("INFO", f"Querying Qdrant for user input: {user_input}")
-    qdrant_client_instance = QdrantClient(qdrant_host, port=qdrant_port)
+    qdrant_client_instance = QdrantClient(
+        qdrant_host, port=qdrant_port, api_key=qdrant_api_key, https=False
+    )
     query_vector = generate_embedding_sync(embedding_model, user_input)
     search_params = models.SearchParams(hnsw_ef=hnsw_ef, exact=False)
     search_result = qdrant_client_instance.search(
@@ -149,7 +80,7 @@ def query_qdrant_knowledge_base(
     custom_log("INFO", f"Received {len(search_result)} results from Qdrant")
 
     str_knowledge_base = ""
-    # fastapi_url = os.environ.get('FAST_API_URL', 'http://localhost:8000')
+    # fastapi_url = os.environ.get('BOTRUN_ASK_FOLDER_FAST_API_URL', 'http://localhost:8000')
     pdf_list = []
     for idx, hit in enumerate(search_result, start=1):
         google_file_id = hit.payload.get(google_file_id_field, "")
@@ -257,6 +188,8 @@ def query_qdrant_and_llm(
     sheet_name_field,
     file_upload_date_field,
     include_ref_page: bool = True,
+    chat_history: List[Dict] = [],
+    qdrant_api_key="",
 ):
     str_message = get_message_for_rag_plus_completion_call(
         qdrant_host,
@@ -275,8 +208,14 @@ def query_qdrant_and_llm(
         ori_file_name_field,
         sheet_name_field,
         file_upload_date_field,
+        qdrant_api_key=qdrant_api_key,
+    )
+    return completion_call(
+        chat_model,
+        str_message,
+        include_ref_page=include_ref_page,
+        chat_history=chat_history,
     )
-    return completion_call(chat_model, str_message, include_ref_page)
 
 
 def get_message_for_rag_plus_completion_call(
@@ -296,6 +235,7 @@ def get_message_for_rag_plus_completion_call(
     ori_file_name_field,
     sheet_name_field,
     file_upload_date_field,
+    qdrant_api_key="",
 ) -> str:
     str_knowledge_base = query_qdrant_knowledge_base(
         qdrant_host,
@@ -313,13 +253,14 @@ def get_message_for_rag_plus_completion_call(
         ori_file_name_field,
         sheet_name_field,
         file_upload_date_field,
+        qdrant_api_key=qdrant_api_key,
     )
     if not notice_prompt:
         notice_prompt = DEFAULT_NOTICE_PROMPT
     str_message = f"""
     <知識庫RAG搜索到的內容>
     {str_knowledge_base}
-    </知識庫RAG搜索到的內容>
+    </知識庫RAG搜索到的容>
 
     <回答時請妳注意>
     {notice_prompt}
@@ -332,6 +273,60 @@ def get_message_for_rag_plus_completion_call(
     return str_message
 
 
+def get_reference_content_from_messages(messages: List[Dict]) -> List[Reference]:
+    # Find the relevant content
+    content = None
+    for message in reversed(messages):
+        if (
+            message["role"] in ["user", "system"]
+            and "<a-rag-file>" in message["content"]
+            and "</a-rag-file>" in message["content"]
+        ):
+            content = message["content"]
+            break
+
+    if not content:
+        return []
+
+    # Extract and parse <a-rag-file> blocks
+    rag_blocks = re.findall(r"<a-rag-file>(.*?)</a-rag-file>", content, re.DOTALL)
+    references = []
+
+    for block in rag_blocks:
+        ori_file_name = re.search(r"<ori_file_name>(.*?)</ori_file_name>", block)
+        page_number = re.search(r"<page_number>(.*?)</page_number>", block)
+        sheet_name = re.search(r"<sheet_name>(.*?)</sheet_name>", block)
+        google_file_id = re.search(r"<google_file_id>(.*?)</google_file_id>", block)
+
+        if ori_file_name and google_file_id:
+            reference = Reference(
+                file_name=ori_file_name.group(1),
+                file_link=f"https://drive.google.com/file/d/{google_file_id.group(1)}/view",
+                page_numbers=[page_number.group(1)] if page_number else [],
+                sheet_names=[sheet_name.group(1)] if sheet_name else [],
+            )
+            references.append(reference)
+
+    # Combine references with the same file_link
+    combined_references = {}
+    for ref in references:
+        if ref.file_link in combined_references:
+            existing_ref = combined_references[ref.file_link]
+            existing_ref.page_numbers.extend(ref.page_numbers)
+            existing_ref.sheet_names.extend(ref.sheet_names)
+        else:
+            combined_references[ref.file_link] = ref
+
+    # Remove duplicates and sort page_numbers and sheet_names
+    for ref in combined_references.values():
+        ref.page_numbers = sorted(
+            set(map(int, filter(lambda x: x.isdigit(), ref.page_numbers)))
+        )
+        ref.sheet_names = sorted(set(ref.sheet_names))
+
+    return list(combined_references.values())
+
+
 def rag_plus_completion_call(
     model: str,
     messages: List = [],
@@ -343,7 +338,9 @@ def rag_plus_completion_call(
     """
     try:
         # custom_log('INFO', f"Messages prepared: {messages}")
+        # print(f"rag_plus_completion_call Messages prepared: {messages}")
         generator = filtered_stream_with_hidden_content(model, messages)
+        lst_references = get_reference_content_from_messages(messages)
         reference_content = ""
         # custom_log('INFO', "Starting iteration over generator")
         # 迭代生成器直到倒數第二個值
@@ -363,17 +360,16 @@ def rag_plus_completion_call(
                 break  # 如果chunk不是字符串，跳出循環
 
         try:
-            custom_log("INFO", f"Reference content: {reference_content}")
-            if reference_content.startswith("```"):
-                contents = reference_content.split("\n")
-                # join the contents except first and last
-                reference_content = "\n".join(contents[1:-1])
-            lst_refs = json.loads(reference_content)["references"]
-            custom_log("INFO", f"References parsed: {lst_refs}")
-            generated_page_name = generate_pdf_page_to_image(lst_refs)
-            custom_log("INFO", f"Generated reference page: {generated_page_name}")
+            # custom_log("INFO", f"Reference content: {reference_content}")
+            # if reference_content.startswith("```"):
+            #     contents = reference_content.split("\n")
+            #     reference_content = "\n".join(contents[1:-1])
+            # lst_refs = json.loads(reference_content)["references"]
+            # custom_log("INFO", f"References parsed: {lst_refs}")
+            # generated_page_name = generate_pdf_page_to_image(lst_refs)
+            # custom_log("INFO", f"Generated reference page: {generated_page_name}")
             generate_ref = generate_ref_section(
-                lst_refs, generated_page_name, include_ref_page
+                lst_references=lst_references,
             )
             custom_log("INFO", f"Generated generate_ref: {generate_ref}")
             yield ModelResponse(
@@ -400,9 +396,12 @@ def rag_plus_completion_call(
         print(f"query_qdrant.py, completion_call, exception: {e}")
 
 
-def completion_call(model, message, include_ref_page: bool = True):
+def completion_call(
+    model, message, include_ref_page: bool = True, chat_history: List[Dict] = []
+):
     # custom_log('INFO', f"Entering completion_call with model: {model}, message: {message}")
-    messages = [{"content": message, "role": "user"}]
+    messages = chat_history + [{"content": message, "role": "user"}]
+    lst_references = []
     # return rag_plus_completion_call(model, messages)
     try:
         response = rag_plus_completion_call(
@@ -419,6 +418,17 @@ def completion_call(model, message, include_ref_page: bool = True):
 
 
 def generate_ref_section(
+    lst_references: List[Reference],
+) -> str:
+    if len(lst_references) == 0:
+        return ""
+    ref_text = "\n參考來源"
+    for ref in lst_references:
+        ref_text += f"\n- {ref.to_md()}"
+    return ref_text
+
+
+def generate_ref_section_old(
     lst_refs: List[Dict], generated_page_name: str, include_ref_page: bool
 ) -> str:
     if len(lst_refs) == 0:
@@ -580,6 +590,7 @@ def query_qdrant_and_llm_print(
     ori_file_name_field="ori_file_name",
     sheet_name_field="sheet_name",
     file_upload_date_field="file-upload-date",
+    qdrant_api_key="",
 ):
     # custom_log('INFO', f"Starting query_qdrant_and_llm for user input: {user_input}")
     for fragment in query_qdrant_and_llm(
@@ -600,6 +611,7 @@ def query_qdrant_and_llm_print(
         ori_file_name_field,
         sheet_name_field,
         file_upload_date_field,
+        qdrant_api_key=qdrant_api_key,
     ):
         print(fragment, end="")
         sys.stdout.flush()
