@@ -97,10 +97,13 @@ def query_qdrant_knowledge_base_keyword(
     sheet_name_field="sheet_name",
     file_upload_date_field="file-upload-date",
     prompt_keyword="auto",
+    qdrant_api_key="",
 ) -> str:
     try:
         custom_log("INFO", f"Querying Qdrant Keyword for user input: {user_input}")
-        qdrant_client_instance = QdrantClient(qdrant_host, port=qdrant_port)
+        qdrant_client_instance = QdrantClient(
+            qdrant_host, port=qdrant_port, api_key=qdrant_api_key, https=False
+        )
 
         # 擷取使用者問題中查詢用的關鍵字
         # Query by keywords roy added
@@ -136,7 +139,7 @@ def query_qdrant_knowledge_base_keyword(
         # custom_log('INFO', f"Received {len(search_result)} results from Qdrant")
 
         str_knowledge_base = ""
-        # fastapi_url = os.environ.get('FAST_API_URL', 'http://localhost:8000')
+        # fastapi_url = os.environ.get('BOTRUN_ASK_FOLDER_FAST_API_URL', 'http://localhost:8000')
         pdf_list = []
 
         # 列印 response 顯示其內容，用於確定結構
