@@ -1,6 +1,6 @@
 from botrun_ask_folder.services.storage.storage_store import StorageStore
 from botrun_ask_folder.services.storage.storage_cs_store import StorageCsStore
-from botrun_ask_folder.services.storage.storage_client import StorageStore
+from botrun_ask_folder.services.storage.storage_client import StorageClient
 from botrun_ask_folder.services.storage.fast_api_storage_client import (
     FastAPIStorageClient,
 )
@@ -10,5 +10,5 @@ def storage_store_factory() -> StorageStore:
     return StorageCsStore()
 
 
-def storage_client_factory() -> StorageStore:
+def storage_client_factory() -> StorageClient:
     return FastAPIStorageClient()
